/*
 * CenterPass.hpp
 *
 * class performing center-pass filtering of monochromatic images
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_CENTERPASS_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_CENTERPASS_HPP_FILE

/* public header */

#include "Graphics/Filter/GeneralRaster.hpp"
#include "Graphics/Filter/GeneralRasterTransformation.hpp"


namespace Graphics
{
namespace Filter
{

// helper object for computing final pixel value
class CenterPassProcessor
{
public:
  inline CenterPassProcessor(const color min, const color max,
                             const color defaultMinColor,
                             const color defaultMaxColor ):
    _min(min),
    _max(max),
    _defMin(defaultMinColor),
    _defMax(defaultMaxColor)
  {
  }

  inline PixelMono operator()(const Position &pos, const RasterMono &r) const
  {
    const PixelMono &in=r.getPixel(pos);
    if(in.get()<_min)    // color is below range?
      return PixelMono(_defMin);
    if(in.get()>_max)    // color is above range?
      return PixelMono(_defMax);
    // color matches range
    return in;
  }

private:
  const color _min;
  const color _max;
  const color _defMin;
  const color _defMax;
}; // class CenterPassProcessor


//
// removes everything that does not belong to [min;max] range.
//
class CenterPass: public GeneralRaster<PixelMono, CenterPassProcessor>
{
public:
  //
  // min             - minimum value of color to preserve it
  // max             - maximum value of color to preserve it
  // defaultMinColor - color to use for pixel when it is smaller
  //                   than min
  // defaultMaxColor - color to use for pixel when it is grater
  //                   than max
  //
  CenterPass(const color min, const color max,
             const color defaultMinColor=0,
             const color defaultMaxColor=0 ):
    GeneralRaster<PixelMono, CenterPassProcessor>(
        CenterPassProcessor(min, max, defaultMinColor, defaultMaxColor) )
  {
  }
}; // class CenterPass


//
// center-pass transformation
//
class CenterPassTransformation: public
          GeneralRasterTransformation<PixelMono, CenterPassProcessor>
{
private:
  typedef GeneralRasterTransformation<PixelMono, CenterPassProcessor> TBase;

public:
  CenterPassTransformation(       TBase::TShrRaster out,
                            const TBase::TShrRaster in,
                            const color min, const color max,
                            const color defaultMinColor=0,
                            const color defaultMaxColor=0 ):
    TBase( CenterPassProcessor(min,max, defaultMinColor,defaultMaxColor),
          out, in )
  {
  }
}; // class CenterPassTransformation

} // namespace Filter
} // namespace Graphics

#endif

