/*
 * General.hpp
 *
 * generic filter operation - iterates throught
 * all pixels of raster/layer and performs user-defined
 * operation on each of them.
 *
 * NOTE: order in which pixels will be processed
 *       is NOT defined, and therefore it should
 *       not be assumed anyhow (it might change
 *       in a future).
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_GENERAL_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_GENERAL_HPP_FILE

/* public header */

#include <boost/noncopyable.hpp>

#include "Graphics/Position.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

//
// the TPixOp function object should have defined
// following operator:
//
//   TImage::PixelType operator()(const Position &pos, const TImage &in) const
//


//
// generic class - input and output types may differ
//
// TImageOut/TImageIn - static polimorphic. this can be
//                      either Raster<> or RasterLayer<>.
//
template <typename TImageOut, typename TImageIn, typename TPixOp>
class GeneralAny: private boost::noncopyable
{
public:
  GeneralAny(const TPixOp &op):
    _op(op)
  {
  }

  void filter(TImageOut &out, const TImageIn &in) const
  {
    if( in.size()!=out.size() )
      throw Graphics::Exception("GeneralAny::filter(): "
                                "input and output images sizes doesn't match");
    // iterate throught image
    const unsigned int sizeX=in.size().x();
    const unsigned int sizeY=in.size().y();
    for(unsigned int y=0; y<sizeY; ++y)
      for(unsigned int x=0; x<sizeX; ++x)
      {
        const Position pos(x,y);
        out.setPixel( pos, _op(pos, in) );
      }
  }

  inline const TPixOp &getPixOp(void) const
  {
    return _op;
  }

private:
  const TPixOp _op;
}; // class GeneralAny


//
// class version assuming having the same in/out
// raster types.
//
template <typename TImage, typename TPixOp>
class General: public GeneralAny<TImage, TImage, TPixOp>
{
public:
  General(const TPixOp &op):
    GeneralAny<TImage, TImage, TPixOp>(op)
  {
  }
}; // class General

} // namespace Filter
} // namespace graphics

#endif

