/*
 * Median.hpp
 *
 * median filter implementation
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_MEDIAN_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_MEDIAN_HPP_FILE

/* public header */

#include <algorithm>

#include "Graphics/Filter/GeneralLayer.hpp"
#include "Graphics/Filter/GeneralLayerTransformation.hpp"


namespace Graphics
{
namespace Filter
{

// main processing unit - computes Median for single pixel
template <typename TPixel>
struct MedianProcessor
{
  typedef          RasterLayer<TPixel>     TRasterLayer;
  typedef typename TRasterLayer::ColorType ColorType;

  inline ColorType operator()(const Position     &pos,
                              const TRasterLayer &in) const
  {
    const size_t x=pos.x();
    const size_t y=pos.y();

    // set center pixel to be default one
    const ColorType &pDef=in.getPixel(pos);
    ColorType pIn[9]={
                       in.getPixel( Position(x-1,y-1), pDef ),
                       in.getPixel( Position(x  ,y-1), pDef ),
                       in.getPixel( Position(x+1,y-1), pDef ),

                       in.getPixel( Position(x-1,y  ), pDef ),
                       in.getPixel( Position(x  ,y  ), pDef ),
                       in.getPixel( Position(x+1,y  ), pDef ),

                       in.getPixel( Position(x-1,y+1), pDef ),
                       in.getPixel( Position(x  ,y+1), pDef ),
                       in.getPixel( Position(x+1,y+1), pDef ),
                     };
    // sort pixels in ascending order
    std::sort( pIn, pIn+sizeof(pIn)/sizeof(pIn[0]) );
    return pIn[4];    // return median value
  }
}; // struct MedianProcessor


//
// processing unit to use for median filtering
//
template <typename TPixel>
class Median: public GeneralLayer< TPixel, MedianProcessor<TPixel> >
{
public:
  typedef GeneralLayer<TPixel, MedianProcessor<TPixel> > TBase;

  Median(void):
    TBase( (MedianProcessor<TPixel>()) )
  {
  }
}; // class Median


//
// median filter transformation
//
template <typename TPixel>
class MedianTransformation: public
        GeneralLayerTransformation<TPixel, MedianProcessor<TPixel> >
{
private:
  typedef GeneralLayerTransformation<TPixel,
                    MedianProcessor<TPixel> > TBase;
  typedef RasterLayer<TPixel>                 TRasterLayer;

public:
  MedianTransformation(      TRasterLayer out,
                       const TRasterLayer in):
    TBase( (MedianProcessor<TPixel>()), out, in )
  {
  }
}; // class MedianTransformation

} // namespace Filter
} // namespace Graphics

#endif

