/*
 * SobelEdgeProcessor.hpp
 *
 * Sobel edge processor. it is preaty big, and so
 * declared in separate file.
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_SOBELEDGEPROCESSOR_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_SOBELEDGEPROCESSOR_HPP_FILE

/* public header */

#include <boost/type_traits/is_integral.hpp>

#include "Graphics/RasterLayer.hpp"


namespace Graphics
{
namespace Filter
{

namespace
{
// this is short metaprogram to select more general
// type for given pixel color type.

// non-integral:
template<bool isIntegral>
struct GenericType
{ typedef double type; };

// integral:
template<>
struct GenericType<true>
{ typedef long   type; };

}

// main processing unit - computes sobel filter for single pixel
template<typename TPixel>
class SobelEdgeProcessor
{
public:
  typedef          RasterLayer<TPixel> TRasterLayer;
  typedef typename TPixel::ColorType   ColorType;
  typedef typename GenericType< boost::is_integral<ColorType>::value >::type
                                       GenericColorType;

  SobelEdgeProcessor(void):
    _maxColorValue( std::numeric_limits<ColorType>::max() )
  {
  }

  // this is entry point used by SobelEdge
  inline ColorType operator()(const Position     &pos,
                              const TRasterLayer &in) const
  {
    const GenericColorType resGx=computeGx(pos, in);
    const GenericColorType resGy=computeGy(pos, in);
    return sumElems(resGx, resGy);  // implicite conv. int -> uchar
  }

private:
  inline ColorType sumElems(GenericColorType c1,
                            GenericColorType c2) const
  {
    const GenericColorType max=_maxColorValue;
    const GenericColorType v  =abs(c1)+abs(c2);
    return std::min(max, v);
  }

  //
  // mask:
  //
  //  -1 0 +1
  //  -2 0 +2
  //  -1 0 +1
  //
  inline GenericColorType computeGx(const Position     &pos,
                                    const TRasterLayer &in  ) const
  {
    GenericColorType res   =0;
    const ColorType  defPix=in.getPixel(pos);

    res+= -1*in.getPixel( Position(pos.x()-1, pos.y()-1), defPix );
    res+= -2*in.getPixel( Position(pos.x()-1, pos.y()+0), defPix );
    res+= -1*in.getPixel( Position(pos.x()-1, pos.y()+1), defPix );

    // these are zeros
    //res+= +0*in.getPixel( Position(pos.x()+0, pos.y()-1), defPix );
    //res+= +0*in.getPixel( Position(pos.x()+0, pos.y()+0), defPix );
    //res+= +0*in.getPixel( Position(pos.x()+0, pos.y()+1), defPix );

    res+= +1*in.getPixel( Position(pos.x()+1, pos.y()-1), defPix );
    res+= +2*in.getPixel( Position(pos.x()+1, pos.y()+0), defPix );
    res+= +1*in.getPixel( Position(pos.x()+1, pos.y()+1), defPix );

    return res;
  }

  //
  // mask:
  //
  //  +1 +2 +1
  //   0  0  0
  //  -1 -2 -1
  //
  inline GenericColorType computeGy(const Position     &pos,
                                    const TRasterLayer &in  ) const
  {
    GenericColorType res   =0;
    const ColorType  defPix=in.getPixel(pos);

    // zeros are commented out

    res+= +1*in.getPixel( Position(pos.x()-1, pos.y()-1), defPix );
    //res+= +0*in.getPixel( Position(pos.x()-1, pos.y()+0), defPix );
    res+= -1*in.getPixel( Position(pos.x()-1, pos.y()+1), defPix );

    res+= +2*in.getPixel( Position(pos.x()+0, pos.y()-1), defPix );
    //res+= +0*in.getPixel( Position(pos.x()+0, pos.y()+0), defPix );
    res+= -2*in.getPixel( Position(pos.x()+0, pos.y()+1), defPix );

    res+= +1*in.getPixel( Position(pos.x()+1, pos.y()-1), defPix );
    //res+=  0*in.getPixel( Position(pos.x()+1, pos.y()+0), defPix );
    res+= -1*in.getPixel( Position(pos.x()+1, pos.y()+1), defPix );

    return res;
  }

  ColorType _maxColorValue;
}; // struct SobelFilterProcessor

} // namespace Filter
} // namespace Graphics

#endif

