/*
 * General.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_HISTOGRAM_GENERAL_HPP_FILE
#define INCLUDE_GRAPHICS_HISTOGRAM_GENERAL_HPP_FILE

/* public header */

#include <cassert>
#include <string.h>

#include "Graphics/RasterLayer.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Histogram
{

//
// class representing histogram of the image layer
//
template <typename TPixel, size_t valuesMax>
class General
{
public:
  typedef RasterLayer<TPixel>        TRasterLayer;
  typedef typename TPixel::ColorType ColorType;

  explicit General(const TRasterLayer &rl):
    _valuesMax(valuesMax)
  {
    // zero table at the begin
    memset(_h, 0, sizeof(_h));

    // compute histogram
    const Size   &size=rl.size();
    const size_t  dimX=size.x();
    const size_t  dimY=size.y();
    for(unsigned int y=0; y<dimY; ++y)
      for(unsigned int x=0; x<dimX; ++x)
      {
        const Position   pos(x,y);
        const ColorType &c=rl.getPixel(pos);
        ++_h[c];
      }
  }

  inline const size_t &operator[](const size_t &pos) const
  {
    assert(pos<valuesMax);
    return _h[pos];
  }

  // smooth histogram
  void smooth(void)
  {
    const General<TPixel,valuesMax> hTmp(*this);    // temporary copy

    assert(_valuesMax>=5);

    // first two need to be handled seraretly
    _h[0]=(hTmp[0]+hTmp[1]+hTmp[2]        )/3;
    _h[1]=(hTmp[0]+hTmp[1]+hTmp[2]+hTmp[3])/4;

    // main processing is schematic
    for(size_t i=0+2; i<_valuesMax-2; ++i)
      _h[i]=(hTmp[i-2]+hTmp[i-1]+hTmp[i]+hTmp[i+1]+hTmp[i+2])/5;

    // last two need to be handled separetly
    const size_t m=_valuesMax-1;
    _h[m-1]=(hTmp[m-3]+hTmp[m-2]+hTmp[m-1]+hTmp[m])/4;
    _h[m-0]=(          hTmp[m-2]+hTmp[m-1]+hTmp[m])/3;
  }

  // integrate histogram
  void integrate(void)
  {
    for(size_t i=1; i<_valuesMax; ++i)
      _h[i]+=_h[i-1];
  }

private:
  size_t _h[valuesMax]; // histogram of given layer
  size_t _valuesMax;    // remember max values to use in .cpp
}; // class General

} // namespace Histogram
} // namespace Graphics

#endif

