/*
 * Generic.hpp
 *
 * generic image pyramid implementation. it should be
 * derived from to create specific pyramid based
 * on resizer that will be held in upper class.
 *
 * to insert recompute pyramid swap content of
 * first image by calling getFirst() and update.
 * then call compute().
 *
 */
#ifndef INCLUDE_GRAPHICS_PYRAMID_GENERIC_HPP_FILE
#define INCLUDE_GRAPHICS_PYRAMID_GENERIC_HPP_FILE

/* public header */

#include <vector>
#include <boost/noncopyable.hpp>
#include <cassert>

#include "Graphics/SharedRaster.hpp"
#include "Graphics/Scale/Resizer.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Pyramid
{

template<typename TPixel>
class Generic: private boost::noncopyable
{
public:
  typedef Graphics::Raster<TPixel>         TRaster;
  typedef Graphics::SharedRaster<TPixel>   TShrRaster;

  virtual ~Generic(void)
  {
  }

  // computes pyramid - user defines what does it mean
  virtual void compute(void) = 0;

  // return raster for a given level of pyramid
  inline TShrRaster get(size_t h)
  {
    if( h>=_h )
      throw Graphics::Exception("Pyramid::Generic::get() "
                                "specified index out of bound");
    return _pyramid[h];
  }
  // call this insetead of raw get(0) if you want
  // to edit image base of pyramid.
  inline TShrRaster getFirst(void)
  {
    return get(0);
  }

  // returns pyramid height
  inline size_t height(void) const
  {
    return _h;
  }

protected:
  //
  // creates image pyramid using user specified
  // scaling funciton.
  //
  // size    - size of initial image (begin of pyramid)
  // h       - heigth of whole pyramid (i.e. how many
  //           levels shoud it have)
  // scaleFactor - tels how many times image should be scaled.
  //               default value (2) means 1/2 scale.
  //
  Generic(const Size   &size,
          const size_t  h,
          const double  scaleFactor=2):
    _size(size),
    _h(h),
    _scaleFactor(scaleFactor)
  {
    if(_h<=1)               // pyramid with <2 levels does not make sens.
      throw Graphics::Exception("Pyramid::Generic(): "
                                "pyramid height should be >1");
    if(_scaleFactor<=1)     // images must became smaller to make pyramid
      throw Graphics::Exception("Pyramid::Generic(): "
                                "scale factor must be >1 (i.e. image "
                                "size must be descending)");

    // create all levels of pyramid
    Size s=size;                            // init size
    _pyramid.push_back( TShrRaster(s) );    // add first level
    for(size_t i=1; i<h; ++i)               // skip first level as it has
    {                                       // been already added
      // compute new size
      const unsigned int x=static_cast<unsigned int>( s.x()/scaleFactor );
      const unsigned int y=static_cast<unsigned int>( s.y()/scaleFactor );
      // check if it does make any sense...
      if(x<=2 || y<=2)
        throw Graphics::Exception("Pyramid::Generic(): "
                                  "too many levels of pyramid for "
                                  "given pyramid initial size");
      // assing new size
      s=Size(x,y);

      // append new level in pyramid
      _pyramid.push_back( TShrRaster(s) );
    }
  }

  // replace given image with different one
  void replace(TShrRaster r, size_t h)
  {
    if( get(h)->size()!=r->size() ) // size mismatch?
      throw Graphics::Exception("Pyramid::Generic:replace(): "
                                "image sizes doesn't match");
    // this is save (get(h) didn't throw)
    assert( get(h).get()!=NULL );   // shouldn't throw is enought
    _pyramid[h]=r;
  }

private:
  const Size               _size;
  const size_t             _h;
  std::vector<TShrRaster>  _pyramid;
  const double             _scaleFactor;
}; // class Generic

} // namespace Pyramid
} // namespace Graphics

#endif

