/*
 * Raster.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_RASTER_HPP_FILE
#define INCLUDE_GRAPHICS_RASTER_HPP_FILE

/* public header */

#include <vector>
#include <cassert>

#include "Graphics/Pixel.hpp"
#include "Graphics/Size.hpp"
#include "Graphics/Position.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{

//
// forward declaration of friend-class
//
template <typename TPix>
class RasterLayer;

//
// class representing an image.
//
// note: after initialization raster has random content
//
template<class TPix>
class Raster
{
public:
  // this is required in generic cases when
  // working with other templates.
  typedef TPix PixelType;

  explicit Raster(const Graphics::Size &size):
                _size(size),
                _r( _size.y() )
  {
    for(unsigned int y=0; y<_size.y(); ++y)
      _r[y].resize( _size.x() );
  }

  inline const Graphics::Size &size(void) const
  {
    return _size;
  }

  inline bool inRange(const Graphics::Position &pos) const
  {
    return _size.inRange(pos);
  }

  // return reference to const pixel
  inline const TPix &getPixel(const Graphics::Position &pos) const
  {
    assert( inRange(pos) );
    return _r[pos.y()][pos.x()];
  }
  // returns pixel from a given position, or defaultPix
  // of given position is out of range (usefull for filtering)
  inline const TPix &getPixel(const Graphics::Position &pos,
                              const TPix               &defaultPixel) const
  {
    if( inRange(pos) )
      return getPixel(pos);
    // if pixel is out of range, return default value
    return defaultPixel;
  }
  // return pixel or throw if position is out of range
  inline const TPix &getPixelSec(const Graphics::Position &pos) const
  {
    if( !inRange(pos) )
      throw Graphics::Exception("Raster: unable to get pixel out of range");
    return getPixel(pos);
  }

  // set new value for pixel
  inline void setPixel(const Graphics::Position &pos, const TPix &pix)
  {
    assert( inRange(pos) );
    _r[pos.y()][pos.x()]=pix;
  }
  inline void setPixelSec(const Graphics::Position &pos, const TPix &pix)
  {
    if( !inRange(pos) )
      throw Graphics::Exception("Raster: unable to set pixel out of range");
    setPixel(pos, pix);
  }

  void fill(const TPix &pix)
  {
    for(typename Image::iterator itY=_r.begin(); itY!=_r.end(); ++itY)
    {
      Row &row=*itY;    // make code more readable
      for(typename Row::iterator itX=row.begin(); itX!=row.end(); ++itX)
        *itX=pix;       // write new value of pixel
    }
  }

private:
  //
  // this is not neccesairly required but using this
  // in RasterLayer, in particular situation is good
  // performance improvment.
  //
  friend class Graphics::RasterLayer<TPix>;
  TPix &getWritablePixelRef(const Position &pos)
  {
    return _r[pos.y()][pos.x()];
  }

  // delcarations belos whould NOT be used in RasterLayer.
  typedef std::vector<TPix> Row;
  typedef std::vector<Row>  Image;

  Graphics::Size _size;     // for better performance
  Image          _r;        // internal representation for raster [y;x]
}; // class Raster



//
// here are some commonly used definitions
//
typedef Graphics::Raster<Graphics::PixelRGB>    RasterRGB;
typedef Graphics::Raster<Graphics::PixelMono>   RasterMono;
typedef Graphics::Raster<Graphics::PixelBinary> RasterBinary;

} // namespace Graphics

#endif

