/*
 * RasterLayer.hpp
 *
 * representation of single layer of a raster.
 * this is only decorator, so that changes made to
 * "layer" applies to "real" image.
 *
 * RasterLayer is a thin wrapper - it has no computation
 * overhead comparing with Raster itself.
 *
 */
#ifndef INCLUDE_GRAPHICS_RASTERLAYER_HPP_FILE
#define INCLUDE_GRAPHICS_RASTERLAYER_HPP_FILE

/* public header */

#include "System/Enum.hpp"
#include "Graphics/Raster.hpp"
#include "Graphics/Pixel.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{

//
// generic raster-layer defining class
//
template <typename TPixel>
class RasterLayer
{
public:
  typedef          Raster<TPixel>    TRaster;
  typedef typename TPixel::ColorType ColorType;

  //   r       - input raster to take layer from
  //   layerNo - layer number to use
  RasterLayer(TRaster *r, size_t layerNo):
    _r(r),
    _layerNo(layerNo)
  {
    if( _layerNo>=TPixel::dims )    // non-existing layer?
      throw Graphics::Exception("RasterLayer::RasterLayer(): "
                                "layer of given number does not exist");
  }

  //
  // this is Raster<> API, which selects only 1 layer
  //

  inline const Size &size(void) const
  {
    return _r->size();
  }

  inline bool inRange(const Position &pos) const
  {
    return _r->inRange(pos);
  }

  inline const ColorType &getPixel(const Position &pos) const
  {
    return _r->getPixel(pos).layer(_layerNo);
  }

  inline const ColorType &getPixel(const Position  &pos,
                                   const ColorType &defaultColor) const
  {
    if( inRange(pos) )
      return getPixel(pos);
    // if pixel is out of range, return default value
    return defaultColor;
  }

  inline const ColorType &getPixelSec(const Position &pos) const
  {
    return _r->getPixelSec(pos).layer(_layerNo);
  }

  inline void setPixel(const Position &pos, const ColorType &c)
  {
    // here we use 'friend' declaration inside Raster<>
    // class to enchance performance of setting layer
    // value (no need to copy whole object).
    _r->getWritablePixelRef(pos).layer(_layerNo, c);
  }
  inline void setPixelSec(const Position &pos, const ColorType &c)
  {
    if( !inRange(pos) )
      throw Graphics::Exception("RasterLayer::setPixelSec(): "
                                "unable to set pixel out of range");
    setPixel(pos, c);
  }

  void fill(const ColorType &c)
  {
    const Size   s   =_r->size();
    const size_t dimX=s.x();
    const size_t dimY=s.y();
    for(size_t y=0; y<dimY; ++y)
      for(size_t x=0; x<dimX; ++x)
        setPixel( Position(x,y), c );
  }

private:
  TRaster *_r;
  size_t   _layerNo;
}; // class RasterLayer




//
// declaration and create function for RGB
//
typedef RasterLayer<PixelRGB>  RasterLayerRGB;

inline RasterLayerRGB makeRasterLayerRGB(RasterRGB *r, PixelRGB::Layer l)
{
  return RasterLayerRGB( r, l.toInt() );
}

//
// delcaration and create function for Mono
//
typedef RasterLayer<PixelMono> RasterLayerMono;

inline RasterLayerMono makeRasterLayerMono(RasterMono *r)
{
  return RasterLayerMono(r, 0);
}

//
// delcaration and create function for Binary
//
typedef RasterLayer<PixelBinary> RasterLayerBinary;

inline RasterLayerBinary makeRasterLayerBinary(RasterBinary *r)
{
  return RasterLayerBinary(r, 0);
}

} // namespace Graphics

#endif

