/*
 * StorerByExt.t.cpp
 *
 */
#include <tut.h>
#include <string>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <math.h>

#include "Graphics/Storage/StorerByExt.hpp"

namespace Graphics
{
namespace Storage
{

struct StorerByExtTestClass
{
  StorerByExtTestClass(void):
    _testFileName("someTemporaryImageFile_without_extention_now"),
    _r( new RasterRGB( Size(100,100) ) )
  {
    // initialize raster with some data;
    for(unsigned int y=0; y<_r->size().y(); ++y)
      for(unsigned int x=0; x<_r->size().x(); ++x)
      {
        const Position pos(x,y);
        unsigned char  col=(x+y)%256;
        const PixelRGB pix(col, col, col);
        _r->setPixel(pos, pix);
      }
  }
  ~StorerByExtTestClass(void)
  {
    // to be shure that file is removed
    unlink( _lastFileName.c_str() );
  }

  bool testFileNameExist(const std::string &fileName) const
  {
    struct stat s;
    return !( stat( fileName.c_str(), &s )==-1 && errno==ENOENT );
  }

  std::string makeFileName(const char *ext)
  {
    std::string out=_testFileName +
                     std::string(".") +
                     std::string(ext);
    tut::ensure( !testFileNameExist(out) );
    _lastFileName=out;
    return out;
  }

  bool compareRaster(const RasterRGB &r2)
  {
    if( r2.size()!=_r->size() )
      return false;

    const unsigned char errMax=5;   // this might happen due to
                                    // loss on compressed formats
    for(unsigned int y=0; y<_r->size().y(); ++y)
      for(unsigned int x=0; x<_r->size().x(); ++x)
      {
        const Position pos(x,y);
        const PixelRGB p1=_r->getPixel(pos);
        const PixelRGB p2= r2.getPixel(pos);
        tut::ensure( fabs(p1.getR()-p2.getR())<errMax );
        tut::ensure( fabs(p1.getG()-p2.getG())<errMax );
        tut::ensure( fabs(p1.getB()-p2.getB())<errMax );
      }
    return true;
  }

  void testReadWriteForExt(const char *ext)
  {
    const std::string file=makeFileName(ext);
    tut::ensure( !testFileNameExist(file) );    // file should NOT exist yet

    // test writing
    StorerByExt stW(file, _r);
    stW.write();
    tut::ensure(  testFileNameExist(file) );    // file should apear on fs

    // test reading
    boost::shared_ptr<RasterRGB> rRead( new RasterRGB( _r->size() ) );
    StorerByExt stR(file, rRead);
    stR.read();

    // check if read file is equal to writen one
    tut::ensure( compareRaster( *rRead.get() ) );
  }

  std::string                  _lastFileName;
  std::string                  _testFileName;
  boost::shared_ptr<RasterRGB> _r;
};

} // namespace Storage
} // namespace Graphics


namespace tut
{
typedef Graphics::Storage::StorerByExtTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Storage/StorerByExt");
}

using namespace std;
using namespace boost;
using namespace Graphics;
using namespace Graphics::Storage;


namespace tut
{

// try storing as BMP
template<>
template<>
void testObj::test<1>(void)
{
  // NOTE: this doesn't fully work, for unknown reason.
  //       the image can be read/write by lib, but it is not
  //       possible to open it with any external tool.
  //       it looks like a bug in Magick++
  testReadWriteForExt("bmp");
}

// try storing as PNG
template<>
template<>
void testObj::test<2>(void)
{
  testReadWriteForExt("png");
}

// try storing as GIF
template<>
template<>
void testObj::test<3>(void)
{
  testReadWriteForExt("gif");
}

// try storing as JPG
template<>
template<>
void testObj::test<4>(void)
{
  testReadWriteForExt("jpg");
}

// try storing as TIF
template<>
template<>
void testObj::test<5>(void)
{
  testReadWriteForExt("tif");
}

// check if it is possible to declare all storers using
// typedef'ed name from include file.
template<>
template<>
void testObj::test<6>(void)
{
  const string name("someLongNonExistingName.blah");

  StorerBMP(name, _r);
  StorerPNG(name, _r);
  StorerGIF(name, _r);
  StorerJPG(name, _r);
  StorerTIF(name, _r);
}

} // namespace tut

