/*
 * CaptureConfigurator.cpp
 *
 */
#include <vector>
#include <cassert>

#include "Graphics/V4L/API1/SystemAPI.hpp"
#include "Graphics/V4L/API1/CaptureConfigurator.hpp"


using namespace std;
using namespace boost;
using namespace System;


namespace Graphics
{
namespace V4L
{
namespace API1
{

CaptureConfigurator::CaptureConfigurator(shared_ptr<Device> dev):
    Graphics::V4L::CaptureConfigurator(dev)
{
  initDevice();
}


Size CaptureConfigurator::configure(const Size   &newSize,
                                    ColorPalette  cp)
{
  return setFormat(newSize, cp);
}


void CaptureConfigurator::initDevice(void)
{
  struct video_capability cap;

  // check for v4l2
  if( getDevice().ioctl(VIDIOC_QUERYCAP, &cap)==0 )
    throw Exception("CaptureConfigurator::initDevice(): "
                    "using V2L1 with V4L2 driver");

  // get capabilities
  if( getDevice().ioctl(VIDIOCGCAP, &cap)==-1 )
    throw Exception("CaptureConfigurator::initDevice(): "
                    "ioctl(VIDIOCGCAP) (get capabilities) failed");

  // resoltions are:
  // cap.maxwidth;cap.maxheight <-> cap.minwidth;cap.minheight

  // init channels
  vector<struct video_channel> channels(cap.channels);
  for(int i=0; i<cap.channels; ++i)
  {
    memset(&channels[i], 0, sizeof(channels[i]));
    channels[i].channel=i;
    if( getDevice().ioctl(VIDIOCGCHAN, &channels[i])==-1 )
      throw Exception("CaptureConfigurator::initDevice(): "
                      "ioctl(VIDIOCGCHAN) (get channel) failed");
  }

  if( !(cap.type&VID_TYPE_CAPTURE) )
    throw Exception("CaptureConfigurator::initDevice(): "
                    "grabbing not supported");
}


Size CaptureConfigurator::setFormat(const Size   &newSize,
                                    ColorPalette  cp      )
{
  struct video_picture pict;
  struct video_window  rd_win;

  // get picture and window information
  if( getDevice().ioctl(VIDIOCGPICT, &pict)  <0 )
    throw Exception("CaptureConfigurator::setFormat(): "
                    "ioctl(VIDIOCGPICT) (get picture info) failed");
  if( getDevice().ioctl(VIDIOCGWIN,  &rd_win)<0 )
    throw Exception("CaptureConfigurator::setFormat(): "
                    "ioctl(VIDIOCGWIN) (get window info) failed (first)");

  // set palette and depth
  pict.palette=colorPalette2ioctlValue(cp);
  pict.depth  =colorPalettePixelSize(cp);
  // set required picture size
  rd_win.width =newSize.x();
  rd_win.height=newSize.y();

  // apply settings to a driver
  if( getDevice().ioctl(VIDIOCSPICT, &pict)  <0 )
    throw Exception("CaptureConfigurator::setFormat(): "
                    "ioctl(VIDIOCSPICT) (set picture info) failed");
  if( getDevice().ioctl(VIDIOCSWIN,  &rd_win)<0 )
    throw Exception("CaptureConfigurator::setFormat(): "
                    "ioctl(VIDIOCSWIN) (set window info) failed");

  // read info from driver once again, to
  // get size of image that will be captured (choosen by driver)
  if( getDevice().ioctl(VIDIOCGWIN,  &rd_win)<0 )
    throw Exception("CaptureConfigurator::setFormat(): "
                    "ioctl(VIDIOCGWIN) (get window info) failed (second)");
  // return size from driver
  const Size setSize(rd_win.width, rd_win.height);
  return setSize;
}

} // namespace API1
} // namespace V4L
} // namespace Graphics

