/*
 * CaptureConfigurator.cpp
 *
 */
#include <cassert>

#include "Graphics/V4L/API2/SystemAPI.hpp"
#include "Graphics/V4L/API2/CaptureConfigurator.hpp"


using namespace std;
using namespace boost;
using namespace System;


namespace Graphics
{
namespace V4L
{
namespace API2
{

CaptureConfigurator::CaptureConfigurator(shared_ptr<Device> dev):
    Graphics::V4L::CaptureConfigurator(dev)
{
  checkDevice();
  setCropping();
}


Size CaptureConfigurator::configure(const Size &newSize,
                                    ColorPalette cp)
{
  return setFormat(newSize, cp);
}


void CaptureConfigurator::checkDevice(void)
{
  // check device
  struct v4l2_capability cap;
  if( getDevice().ioctl(VIDIOC_QUERYCAP, &cap)==-1 )
  {
    if(errno==EINVAL)
      throw Exception("Capturer::initDevice(): "
                      "not a V4L2 device");
    else
      throw Exception("Capturer::initDevice(): "
                      "ioctl(VIDIOC_QUERYCAP) failed");
  }
  if(!(cap.capabilities & V4L2_CAP_VIDEO_CAPTURE))
    throw Exception("Capturer::initDevice(): "
                    "not a video capture device");
  if(!(cap.capabilities & V4L2_CAP_READWRITE))
    throw Exception("Capturer::initDevice(): "
                    "device does not support I/O");
}


void CaptureConfigurator::setCropping(void)
{
  struct v4l2_cropcap cropcap;
  struct v4l2_crop    crop;
  memset(&cropcap, 0, sizeof(cropcap) );

  cropcap.type=V4L2_BUF_TYPE_VIDEO_CAPTURE;
  if( getDevice().ioctl(VIDIOC_CROPCAP, &cropcap)==0 )
  {
    crop.type=V4L2_BUF_TYPE_VIDEO_CAPTURE;
    crop.c   =cropcap.defrect;    // reset to default
    if( getDevice().ioctl(VIDIOC_S_CROP, &crop)==-1 )
    {
      // Cropping not supported.
    }
  }
  // Errors ignored!
}


Size CaptureConfigurator::setFormat(const Size   &newSize,
                                    ColorPalette  cp)
{
  struct v4l2_format fmt;
  memset(&fmt, 0, sizeof(fmt) );
  fmt.type               =V4L2_BUF_TYPE_VIDEO_CAPTURE;
  fmt.fmt.pix.width      =newSize.x();
  fmt.fmt.pix.height     =newSize.y();
  fmt.fmt.pix.pixelformat=colorPalette2ioctlDataFormat(cp);
  fmt.fmt.pix.field      =V4L2_FIELD_INTERLACED;

  if( getDevice().ioctl(VIDIOC_S_FMT, &fmt)==-1 )
    throw Exception("CapturerFinal::initDevice(): "
                    "first ioctl(VIDIOC_S_FMT) failed");
  // Note VIDIOC_S_FMT may have changed width and height.
  const Size setSize=Size( fmt.fmt.pix.width,
                           fmt.fmt.pix.height );

  // Buggy driver paranoia.
  unsigned int min;
  min = fmt.fmt.pix.width * colorPalettePixelSize(cp);
  if( fmt.fmt.pix.bytesperline<min )
    fmt.fmt.pix.bytesperline = min;

  // Buggy driver paranoia.
  min = fmt.fmt.pix.bytesperline * fmt.fmt.pix.height;
  if( fmt.fmt.pix.sizeimage<min )
    fmt.fmt.pix.sizeimage = min;

  // set all data now
  if( getDevice().ioctl(VIDIOC_S_FMT, &fmt)==-1 )
    throw Exception("CapturerFinal::initDevice(): "
                    "second ioctl(VIDIOC_S_FMT) failed");
  // ensure that data has been set correctly
  if( fmt.fmt.pix.width !=setSize.x() ||
      fmt.fmt.pix.height!=setSize.y()    )
    throw Exception("CapturerFinal::initDevice(): "
                    "driver has changed its own settings - "
                    "this should never happen");

  return setSize;
}

} // namespace API2
} // namespace V4L
} // namespace Graphics

