/*
 * Capturer.cpp
 *
 */
#include <errno.h>
#include <cassert>

#include "Graphics/V4L/Capturer.hpp"
#include "Graphics/V4L/API1/CaptureConfigurator.hpp"
#include "Graphics/V4L/API2/CaptureConfigurator.hpp"

using namespace std;
using namespace boost;

namespace Graphics
{
namespace V4L
{

Capturer::Capturer(shared_ptr<Device>  dev,
                   const Size         &size,
                   ColorPalette        cp):
  _dev(dev),
  _size(size),
  _cp(cp)
{
  // use api acording to choosen/set one
  assert( _dev->getAPIType()!=APIType::AUTO );   // should not happen here
  if( _dev->getAPIType()==APIType::V4L2 )
    _cc.reset( new API2::CaptureConfigurator(_dev) );
  else
    _cc.reset( new API1::CaptureConfigurator(_dev) );

  // check if there was some error
  if( _cc.get()==NULL )
    throw Capturer::Exception("Capturer::Capturer(): "
             "unable to create requested V4L API handler/configurator: "
             "unknown error");

  // force reconfiguration, to make it aviable
  reconfigure(_size, _cp, true);
}


void Capturer::capture(any raster, unsigned int timeout)
{
  switch( _cp.toInt() )
  {
    case ColorPalette::RGB24:
      if( RasterRGB **r=any_cast<RasterRGB*>(&raster) )
      {
        capture(**r, timeout);
        return;
      }
  } // switch(ColorPalette)
  throw Capturer::Exception("Capturer::capture(): "
                  "unknown raster type has been given "
                  "in boost::any call");
}


namespace
{
struct rgb
{
  unsigned char _r;
  unsigned char _g;
  unsigned char _b;
} __attribute__((packed));
} // unnamed namespace

void Capturer::capture(RasterRGB &raster, unsigned int timeout)
{
  if( raster.size()!=_size )
    throw Capturer::Exception("Capturer::capture(): "
                    "output raster size does not match with defined");

  //
  // try reading data
  //
  const unsigned int  secs  =timeout/1000;
  const unsigned int  usecs =(timeout%1000)*1000;
  struct timeval      tvDef ={secs, usecs};
  struct timeval      tv;
  struct timeval     *tv2use=(timeout==0)?NULL:&tv;
  const int           fd    =_dev->getDev();
  for(;;)
  {
    // wait on select for data
    fd_set fds;
    FD_ZERO(&fds);
    FD_SET(fd, &fds);
    tv=tvDef;
    switch( select(fd+1, &fds, NULL, NULL, tv2use) )
    {
      case -1:  // error?
        if(errno==EINTR)
          continue;
        else
          throw Capturer::Exception("Capturer::capture(): select() failed");
        break;  // never reach here

      case  0:  // timeout?
        throw Capturer::Exception("Capturer::capture(): timeout");
        break;  // never reach here

      default:  // got some data!
        break;
    }

    // has data? read data!
    int ret;
    int bufSize=_data.size();
    if( (ret=read(fd, _data.get(), bufSize))!=bufSize )
    {
      if(ret==-1 && errno==EAGAIN)  // shit happens... ;)
        continue;
      throw Capturer::Exception("Capturer::capture(): read() error");
    }
    else
      break;
  } // for(;;)

  //
  // copy data to output buffer
  //
  const struct rgb *ptr=
        reinterpret_cast<const struct rgb*>( _data.get() );
  assert(ptr!=NULL);
  const size_t size_x=_size.x();
  const size_t size_y=_size.y();
  for(unsigned int y=0; y<size_y; ++y)
    for(unsigned int x=0; x<size_x; ++x)
    {
      const struct rgb &tmp=*(ptr + y*size_x+x);
      const PixelRGB    pix(tmp._r, tmp._g, tmp._b);
      const Position    pos(x, y);
      raster.setPixel(pos, pix);
    }
}


void Capturer::reconfigure(const Size &size, ColorPalette cp, bool force)
{
  // if device seems to be already configured properly
  // we may actually exit stright away, unless forced.
  bool hasNewConfig=(cp!=_cp || size!=_size);
  if( !hasNewConfig && !force )
    return;

  // try to configure our device and remember size
  // of image provided by configuration class
  _cp  =cp;
  _size=_cc->configure(size, _cp);
  const size_t bufferSize=_size.x()*_size.y()*colorPalettePixelSize(_cp);
  _data.resize(bufferSize);
}

} // namespace V4L
} // namespace Graphics

