/*
 * Capturer.hpp
 *
 * common interface to be used in user-apps, for video
 * frame capturing regardles of API1 or API2 interface usage.
 *
 */
#ifndef INCLUDE_GRAPHICS_V4L_CAPTURER_HPP_FILE
#define INCLUDE_GRAPHICS_V4L_CAPTURER_HPP_FILE

/* public header */

#include <memory>
#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/scoped_array.hpp>
#include <boost/any.hpp>

#include "Graphics/Raster.hpp"
#include "Graphics/AutoRaster.hpp"
#include "Graphics/V4L/Device.hpp"
#include "Graphics/V4L/CaptureConfigurator.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace V4L
{

// wrapper calss for CapturerAbstract - it automaticaly
// selects proper API, according to what has been provided/choosen
// by Device class
class Capturer: private boost::noncopyable
{
public:
  // exception used by this specific class
  class Exception: public Graphics::Exception
  {
  public:
    Exception(const char *msg):
      Graphics::Exception( std::string("Graphics::V4L::Capturer: ")+
                           std::string(msg) )
    {
    }
    Exception(const std::string &msg):
      Graphics::Exception( std::string("Graphics::V4L::Capturer: ")+
                           msg )
    {
    }
  }; // class Exception

  //
  // dev  - device to be used for capturing
  // size - size of image to capture (NOTE: driver
  //        might require diffrent size and change
  //        it during ioctl()'s - after creating object
  //        ensure the actual size with size() method!)
  // cp   - color palette to be used when capturing (implies
  //        raster type to be used for capturing)
  //
  Capturer(boost::shared_ptr<Device>  dev,
           const Size                &size,
           ColorPalette               cp);

  //
  // captures single image, or throw exception if timeouted.
  // may also write data directly to a given memory place
  // (assuming the picture sizes MATCH).
  //   timeout - given in [ms] (0 for infinit wait)
  //
  template<typename TPixel>
  inline AutoRaster<TPixel> capture(unsigned int timeout=500)
  {
    AutoRaster<TPixel> raster( size() );
    capture( boost::any( raster.get() ), timeout );
    return raster;
  }
  //
  // captures raster image and writes data to object given
  // by user (no new objects will be created).
  //   raster  - raw pointer to raster to work on
  //   timeout - given in [ms] (0 for infinit wait)
  //
  void capture(boost::any raster, unsigned int timeout=500);

  // returns actual size of image that will be caputred.
  inline const Size& size(void) const
  {
    return _size;
  }

private:
  // reconfigurator
  void reconfigure(const Size &size, ColorPalette cp, bool force=false);

  // capturers for given formats (color paletts):
  void capture(RasterRGB &raster, unsigned int timeout);

  //
  // helper class - buffor holder
  //
  class DataBuffer: private boost::noncopyable
  {
  public:
    typedef unsigned char Byte;

    DataBuffer(size_t initSize=0):
      _size(0)          // initial value, before resizing
    {
      resize(initSize);
    }

    size_t resize(size_t newSize)
    {
      // resize?
      if(_size!=newSize)
      {
        _size=newSize;
        if(_size>0)     // resize of reset?
          _data.reset( new Byte[_size] );
        else
          _data.reset(NULL);
      }
      return size();
    }

    inline size_t size(void) const
    {
      return _size;
    }

    inline Byte* get(void)
    {
      return _data.get();
    }
  private:
    boost::scoped_array<Byte> _data;
    size_t                    _size;
  }; // class DataBuffer

  // data used by Capturer
  boost::shared_ptr<Device>          _dev;
  std::auto_ptr<CaptureConfigurator> _cc;
  Size                               _size;
  ColorPalette                       _cp;
  DataBuffer                         _data;
}; // class Capturer

} // namespace V4L
} // namespace Graphics

#endif

